package com.me.brickbuster.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Vector2;
import com.me.brickbuster.BrickBuster;
import net.dermetfan.utils.Pair;

public class Paddle extends Entity {

	public static final Color PADDLE_COLOR = Color.BLACK;
	public static final int DEFAULT_WIDTH = 100;
	public static final int PADDLE_HEIGHT = 10;
	public static final int PADDLE_Y = 15;
	public static final int PADDLE_SPEED = 375;

	private boolean sticky = false;
	private int width = DEFAULT_WIDTH;

	public Paddle(BrickBuster brickBuster) {
		super(brickBuster, BrickBuster.WIDTH / 2, PADDLE_Y);
	}

	@Override
	public void render() {
		getShapeRenderer().begin(ShapeType.Filled);
		getShapeRenderer().setColor(sticky? Color.GRAY : PADDLE_COLOR);
		getShapeRenderer().rect(getX() - width/2, getY(), width, PADDLE_HEIGHT);
		getShapeRenderer().end();
	}

	@Override
	public void update(float dt) {
		if (Gdx.input.isKeyPressed(Input.Keys.LEFT)) {
			if (getX() - width/2 - PADDLE_SPEED * dt < 0) {
				setX(width/2);
				return;
			}
			setX(getX() - PADDLE_SPEED * dt);

			for (Ball ball : getBrickBuster().getBalls()) {
				if (ball.isStuck()) {
					ball.setX(ball.getX() - PADDLE_SPEED * dt);
				}
			}
		}
		if (Gdx.input.isKeyPressed(Input.Keys.RIGHT)) {
			if (getX() + width/2 + PADDLE_SPEED * dt > BrickBuster.WIDTH) {
				setX(BrickBuster.WIDTH - width/2);
				return;
			}
			setX(getX() + PADDLE_SPEED * dt);

			for (Ball ball : getBrickBuster().getBalls()) {
				if (ball.isStuck()) {
					ball.setX(ball.getX() + PADDLE_SPEED * dt);
				}
			}
		}
	}

	public Pair<Vector2, Vector2> getTopEdge() {
		return new Pair<Vector2, Vector2>(
				new Vector2(getX() - width/2, getY() + PADDLE_HEIGHT),
				new Vector2(getX() + width/2, getY() + PADDLE_HEIGHT)
		);
	}

	public int getWidth() {
		return width;
	}

	public boolean isSticky() {
		return sticky;
	}

	public void setSticky(boolean sticky) {
		this.sticky = sticky;
	}

	public void setWidth(int width) {
		this.width = width;
	}
}
